package	jp.co.sds_corp.sqltool;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

public class RowWriterFactory
{
	ToolParam prm = null;

	public RowWriterFactory( ToolParam prm ) {
		this.prm = prm;
	}

	protected PrintStream getStream( String file ) {
		PrintStream ps = null;
		try {
			ps = new PrintStream( new FileOutputStream( file ) );
			System.out.println( "write file [" + file + "]" );
			return ps;
		} catch ( IOException ex ) {
			System.out.println( "can't write file :[" + file + "]" );
		}
		return null;
	}

	private PrintWriter getWriter( OutputStream os ) {
		PrintWriter pw = null;
		String enc = prm.getEncoding();
		OutputStreamWriter ow = null;
		if ( enc == null ) {
			ow = new OutputStreamWriter( os );
		} else {
			try {
				ow = new OutputStreamWriter( os, enc );
			} catch ( UnsupportedEncodingException ue ) {
				System.out.println( "Illegal Encoding [" + enc + "]" );
				ow = new OutputStreamWriter( os );
			}
		}
		pw = new PrintWriter( new BufferedWriter( ow ));
		return pw;
	}

	private PrintWriter getWriter( String file ) {
		PrintWriter pw = null;
		try {
			FileOutputStream fo = new FileOutputStream( file );
			pw = getWriter( fo );
			System.out.println( "write file [" + file + "]" );
		} catch ( IOException ie ) {
			System.out.println( "can't write file :[" + file + "]" );
		}
		return pw;
	}

	public IRowWriter createStdoutWriter() {
		IRowWriter writer = null;
		PrintWriter pw = getWriter( System.out );
		if ( pw != null ) {
			writer = createPrintWriter( pw );
			NullFinisher fin = new NullFinisher( prm, pw );
			writer.setFinisher( fin );
		}
		return writer;
	}

	public IRowWriter createVerticalWriter() {
		IRowWriter writer = null;
		PrintWriter pw = getWriter( System.out );
		if ( pw != null ) {
			writer = new VerticalRowWriter( prm, pw );
			NullFinisher fin = new NullFinisher( prm, pw );
			writer.setFinisher( fin );
		}
		return writer;
	}

	public IRowWriter createPrintWriter( String file ) {
		IRowWriter writer = null;
		PrintWriter pw = getWriter( file );
		if ( pw != null ) {
			writer = createPrintWriter( pw );
		}
		return writer;
	}

	public IRowWriter createPrintWriter( PrintWriter pw ) {
		IRowWriter writer = null;
		if ( pw != null ) {
			if ( prm.getMode() == ToolParam.MODE_HORZ ) {
				writer = new PrintRowWriter( prm, pw );
			} else {
				writer = new VerticalRowWriter( prm, pw );
			}
		}
		return writer;
	}

	public IRowWriter createCsvWriter( PrintStream ps ) {
		IRowWriter writer = null;
		PrintWriter pw = getWriter( ps );
		if ( pw != null ) {
			writer = new CsvRowWriter( prm, pw );
		}
		return writer;
	}

	public IRowWriter createCsvWriter( String file ) {
		IRowWriter writer = null;
		PrintWriter pw = getWriter( file );
		if ( pw != null ) {
			writer = new CsvRowWriter( prm, pw );
		}
		return writer;
	}

	public IRowWriter createMoreWriter() {
		IRowWriter writer = null;
		File file = null;
		PrintWriter pw = null;
		try {
			file = File.createTempFile( "sql.", ".tmp" );
			pw = getWriter( new FileOutputStream( file ));
		} catch ( IOException ex ) {
			System.out.println( "can't create temp file :[" + file + "]" );
		}

		if ( pw != null ) {
			writer = createPrintWriter( pw );
			MoreFinisher fin = new MoreFinisher( prm, pw, file );
			writer.setFinisher( fin );
		}
		return writer;
	}

}
